package esac.archive.lpf.common.constants.client;

import java.util.Calendar;
import java.util.Date;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;


/**
 * Utility class to store String values shared by Web and Data Distribution applications.
 * 
 * @author Maria Henar Sarmiento Carrion Copyright (c) 2013 - European Space Agency
 * 
 */
public class LpfGeneralConstants {

    /**
     * Default constructor, it should not be called.
     */
    protected LpfGeneralConstants() {
        // prevents calls from subclass
        throw new UnsupportedOperationException();
    }

    /**
     * Observation ID length.
     */
    public static final int OBSERVATION_ID_LENGTH = 10;

    /** Prop_end date (some date in the future). */
    public static final Date DEFAULT_PROPRIETARY_END_DATE = getDefaultPropEndDate();

    /**
     * This method returns the default proprietary date.
     * @return a string.
     */
    private static Date getDefaultPropEndDate() {
        Calendar dateinFuture = Calendar.getInstance();
        dateinFuture.set(2999, Calendar.JANUARY, 1, 0, 0, 0);
        return dateinFuture.getTime();
    }

    // ///////////////////////////////////////////////////////////////////////
    // Patterns used to validate RA/DEC format
    // ///////////////////////////////////////////////////////////////////////

    /** Regular expression for white Space. */
    private static final String BLANK = "\\s*";

    private static final String SIGN = "([+-])?";

    /** Regular expression to indicate "Begin with white space". */
    private static final String BEGIN = "^" + BLANK;

    /** Regular expression to indicate "End with white space". */
    private static final String END = BLANK + "$";

    // ///////////////////////////////////////////////////////////////////////
    // Atomic elements.
    // ///////////////////////////////////////////////////////////////////////

    /** Regular expression to express hours. */
    private static final String HOUR = "([0-2]?[0-9])";

    /** Regular expression to express minutes. */
    private static final String MIN = "([0-5]?[0-9])";

    /** Regular expression to express seconds. */
    private static final String SEC = "([0-5]?[0-9]\\.?\\d*)";

    /** Regular expression to express. */
    private static final String SEP2 = "[\\s\\:\\,]+";

    /** Regular expression to express degrees. */
    private static final String DEG = "(\\d{1,2})";

    // /** Regular expression to express. */
    // private static final String DEG_LONG = "(\\d{1,3})";
    //
    // /** Regular expression to express radians. */
    // private static final String RADII = "[\\s\\,]*" + "(?:(\\d+\\.?\\d*)(d|m|s|\\'{1,2})?)?"
    // + BLANK;

    /** Regular expression to express hours as mathematical unit. */
    private static final String HOUR_UNIT = "(?:([0-2]?[0-9])h)?";

    /** Regular expression to express minutes as mathematical unit. */
    private static final String MIN_UNIT = "(?:([0-5]?[0-9])m)?";

    /** Regular expression to express seconds as mathematical unit. */
    private static final String SEC_UNIT = "(?:([0-5]?[0-9]\\.?\\d*)s)?";

    /** Regular expression to express degrees as mathematical unit. */
    private static final String DEG_UNIT = "(?:(\\d{1,2})d)?";

    /** Regular expression to express as mathematical unit. */
    // private static final String DEG_LONG_UNIT = "(?:(\\d{1,3})d)?";

    /** Regular expression to express minutes of arc. */
    private static final String AMIN_UNIT = "(?:([0-5]?[0-9])[m\\'])?";

    /** Regular expression to express as seconds of arc. */
    private static final String ASEC_UNIT = "(?:([0-5]?[0-9]\\.?\\d*)(?:s|\\'{2}))?";

    /** RA format: 1 number without units or any with them. */
    public static final String RA_DECIMAL_PATTERN = BEGIN + "([\\d\\.]+)([dh])?" + END;
    /** RA format: 2 numbers without units or any with them. */
    public static final String RA_TWO_NUMBERS_PATTERN = BEGIN + HOUR + SEP2 + MIN + END;
    /** RA format: 3 numbers without units or any with them. */
    public static final String RA_THREE_NUMBERS_PATTERN = BEGIN + HOUR + SEP2 + MIN + SEP2 + SEC
            + END;
    /** RA format with units. */
    public static final String RA_UNITS_PATTERN = BEGIN + HOUR_UNIT + BLANK + MIN_UNIT + BLANK
            + SEC_UNIT + END;
    /** DEC format: 1 number without units or any with them. */
    public static final String DEC_DECIMAL_PATTERN = BEGIN + SIGN + BLANK + "([\\d\\.]+)d?" + END;
    /** DEC format: 2 numbers without units or any with them. */
    public static final String DEC_TWO_NUMBERS_PATTERN = BEGIN + SIGN + BLANK + DEG + SEP2 + MIN
            + END;
    /** DEC format: 3 numbers without units or any with them. */
    public static final String DEC_THREE_NUMBERS_PATTERN = BEGIN + SIGN + BLANK + DEG + SEP2 + MIN
            + SEP2 + SEC + END;
    /** DEC format with units. */
    public static final String DEC_UNITS_PATTERN = BEGIN + SIGN + DEG_UNIT + BLANK + AMIN_UNIT
            + BLANK + ASEC_UNIT + END;

    /** User fields format. */
    public static final String EMAIL_PATTERN = "^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
            + "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$";

    /** Pattern to define phone numbers. */
    public static final String PHONE_PATTERN = "";

    /** Pattern to define usernames. */
    public static final String USERNAME_PATTERN = "^[a-z0-9_-]{3,15}$";

    /**
     * Password pattern: ( # Start of group (?=.*[a-zA-z]) # lowercase or uppercase letters (?=.*\d)
     * # must contains at least one digit from 0-9 (?=[\\S]+$) # no white spaces along the string .
     * # match anything with previous condition checking {6,20} # length at least 6 characters and
     * maximum of 20 ) # End of group
     */
    public static final String PASSWORD_PATTERN = "((?=.*\\d)(?=.*[a-zA-z])(?=[\\S]+$).{6,20})";

    /**
     * Encryption key used for encrypting data in client-server requests (and backwards)
     * 
     * Important!!! If this key is changed, also the one in the class CryptoUtils.java in the Web
     * client project must be changed to be the same
     */
    public static final byte[] NXSA_TRIPLE_DES_CIPHER_KEY = new byte[] { (byte) 3, (byte) 1,
            (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 4,
            (byte) 1, (byte) 1, (byte) 8, (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 1,
            (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 1, (byte) 3, };

    /**
     * Reg.expression used to validate target names. Starts with a letter or number, then it can
     * contain any word character (\w), a space, a dot, a comma, a minus or a plus sign, and ends
     * with a word character (\w). Note that \w is short for [a-zA-Z_0-9]
     * 
     * Example of target names: 2MASS J00420949+4117456 MCG+07-02-012 m 31 Cl* Trumpler 14 MJ 251
     * 3c218
     * 
     */
    public static final String TARGET_NAME_VALID_PATTERN = "[a-zA-Z0-9][-+\\* .\\w]*[\\w]?";

    /**
     * Datatypes for LPF
     */
    public static final String DATATYPE_AO = "AO";
    public static final String DATATYPE_AOIMAGE = "AOImage";
    public static final String DATATYPE_DOC = "Doc";
    public static final String DATATYPE_SERVDOC = "ServiceDoc";
    public static final String DATATYPE_SW = "SW";
    public static final String DATATYPE_AP = "AP";
    public static final String DATATYPE_PD = "PD";
    public static final String DATATYPE_PRD = "PRD";
    public static final String DATATYPE_TM = "TM";
    public static final String DATATYPE_DDS = "DDS";
    

    public static final String DATATYPE_REPO_SERVDOC = "planning";
    
    public static final String SUBDATATYPE_ISIF = "ISIF";

    public static final String SUBDATATYPE_REPORT = "report";
    
    /**
     * Subdatatypes for LPF AO
     */
    public static enum AO_SUBDATATYPES {
        F, M, C, H, A, I, P
    }

    /**
     * Subdatatypes for LPF SERVICE_DOC
     */
    public static enum SERVICEDOC_SUBDATATYPES {
        ISIF, ITCF, POR, TCHistory, DOR
    }

    /**
     * Subdatatypes for LPF DOC
     */
    public static enum DOC_SUBDATATYPES {
        design, paper, report, tech
    }
    
    /**
     * Character used to separate the different values within a tupla parameter=value1,value2,value3
     */
    public static final String MULTI_VALUE_SEPARATOR = ",";

    /**
     * User roles in LPF.
     * 
     * @author mfbarreiro
     */
    public static enum UserRole {
        DEFAULT(-1), BASE_ANALYST(0), ADVANCED_ANALYST(1), ESA_ANALYSIT(2), POWER(3);

        /** repMap Matrix. */
        private static final Map<Integer, UserRole> roleMap = new HashMap<Integer, UserRole>();

        static {
            for (UserRole type : EnumSet.allOf(UserRole.class)) {
                roleMap.put(type.roleId, type);
            }
        }
        
        /** User role unique identifier. */
        private Integer roleId;

        /**
         * Default constructor.
         * @param identifier role identifier.
         */
        UserRole(final Integer identifier) {
            this.roleId = identifier;
        }

        /**
         * @return the roleId
         */
        public Integer getRoleId() {
            return roleId;
        }

        /**
         * getByIndex().
         * @param index Input tab index.
         * @return CalendarTabView
         */
        public static UserRole getByIndex(final Integer index) {
            return roleMap.get(index);
        }
        
    }
}
